package ru.dbotthepony.mc.otm.block;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityTicker;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.material.Material;
import net.minecraft.world.level.material.MaterialColor;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.energy.CapabilityEnergy;
import net.minecraftforge.energy.EnergyStorage;
import net.minecraftforge.energy.IEnergyStorage;
import org.apache.logging.log4j.core.jmx.Server;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.AndroidCapability;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;

public class BlockAndroidStation extends Block implements EntityBlock {
	public static final ResourceLocation REGISTRY_NAME = new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_station");
	private final VoxelShape SHAPE = Shapes.box(0, 0, 0, 1, 0.5, 1);

	public BlockAndroidStation() {
		super(BlockBehaviour.Properties.of(Material.STONE, MaterialColor.STONE).requiresCorrectToolForDrops().strength(3F, 6.0F));
		this.setRegistryName(REGISTRY_NAME);
	}

	@Override
	public VoxelShape getShape(BlockState p_151964_, BlockGetter p_151965_, BlockPos p_151966_, CollisionContext p_151967_) {
		return SHAPE;
	}

	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos pos, BlockState state) {
		return new BlockAndroidStationEntity(pos, state);
	}

	@Nullable
	@Override
	public <T extends BlockEntity> BlockEntityTicker<T> getTicker(Level level, BlockState state, BlockEntityType<T> type) {
		return level.isClientSide || type != OverdriveThatMatters.ANDROID_STATION_FACTORY ? null : BlockAndroidStationEntity::tick;
	}

	public static class BlockAndroidStationEntity extends BlockEntity {
		private static class AndroidStationEnergyStorage extends EnergyStorage {
			public AndroidStationEnergyStorage(int capacity, int maxReceive, int maxExtract) {
				super(capacity, maxReceive, maxExtract);
			}

			void setEnergy(int energy) {
				this.energy = energy;
			}
		}

		private final AndroidStationEnergyStorage energy = new AndroidStationEnergyStorage(64000, 1024, 1024);

		public BlockAndroidStationEntity(BlockPos p_155229_, BlockState p_155230_) {
			super(OverdriveThatMatters.ANDROID_STATION_FACTORY, p_155229_, p_155230_);
		}

		@Nonnull
		public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
			if (cap == CapabilityEnergy.ENERGY) {
				return LazyOptional.of(() -> energy).cast();
			}

			return super.getCapability(cap, side);
		}

		@Override
		public CompoundTag save(CompoundTag nbt) {
			nbt.putInt("energy_stored", energy.getEnergyStored());
			return super.save(nbt);
		}

		public void load(CompoundTag nbt) {
			super.load(nbt);

			if (nbt.contains("energy_stored"))
				energy.setEnergy(nbt.getInt("energy_stored"));
		}

		void tick() {
			if (!hasLevel())
				return;

			if (energy.getEnergyStored() <= 0)
				return;

			BlockPos pos = getBlockPos();
			List<LivingEntity> entities = getLevel().getEntitiesOfClass(LivingEntity.class, new AABB(pos.getX(), pos.getY(), pos.getZ(), pos.getX() + 1, pos.getY() + 2, pos.getZ() + 1));

			for (LivingEntity ent : entities) {
				LazyOptional<IAndroidCapability> resolver = ent.getCapability(AndroidCapability.CAPABILITY);

				if (resolver.isPresent()) {
					IAndroidCapability capability = resolver.resolve().get();

					int missing = capability.getMissingPowerInt();

					if (missing > 0) {
						int extract = energy.extractEnergy(missing, true);

						if (extract > 0) {
							int received = (int) capability.receiveEnergy(extract, true);

							if (received > 0) {
								energy.extractEnergy(extract, false);
								capability.receiveEnergy(extract, false);
							}
						}
					}
				}
			}
		}

		public static <T extends BlockEntity> void tick(Level level, BlockPos blockPos, BlockState blockState, T t) {
			if (t instanceof BlockAndroidStationEntity tile) {
				tile.tick();
			}
		}
	}
}
